@@ -1,6 +1,6 @@
 package MyPageKit::Common;
 
-# $Id: Common.pm,v 1.19 2002/01/08 16:45:12 borisz Exp $
+# $Id: Common.pm,v 1.11 2001/07/09 02:42:10 tjmather Exp $
 
 use strict;
 
@@ -9,14 +9,11 @@ use vars qw(@ISA);
 
 use Apache::Constants qw(OK REDIRECT DECLINED);
 
-use Digest::MD5 ();
+$__PACKAGE__::secret_md5 = 'you_should_place_your_own_md5_string_here';
+
 use DBI;
-
-no strict 'refs';
-${ __PACKAGE__ . '::secret_md5' } = 'you_should_place_your_own_md5_string_here';
-use strict 'refs';
-
 use MyPageKit::MyModel;
+use Digest::MD5;
 
 sub pkit_dbi_connect {
   # this line should be replaced with a DBI->connect(...) statement
@@ -47,12 +44,7 @@ sub pkit_common_code {
   my $model = shift;
 
   # put code that is common to all pages here
-  my $session = {};
-
-  # only create new session if pkit_admin is set
-  if ($model->pkit_get_session_id || $model->input('pkit_admin')) {
-    $session = $model->session;
-  }
+  my $session = $model->pkit_get_session_id ? $model->session : {};
 
   # for the pagekit.org website, we control the colors based on the
   # values the user selected, stored in the session.
@@ -60,12 +52,6 @@ sub pkit_common_code {
   $model->output(text_color => $session->{'text_color'} || '000000');
   $model->output(bgcolor => $session->{'bgcolor'} || 'dddddd');
   $model->output(mod_color => $session->{'mod_color'} || 'ffffff');
-
-  # toggle on-line editing tools
-  if($model->input('pkit_admin')) {
-    $session->{'pkit_admin'} = $model->input('pkit_admin') eq 'on' ? 1 : 0;
-  }
-  $model->output(pkit_admin => $session->{'pkit_admin'});
 }
 
 sub pkit_auth_credential {
@@ -74,8 +60,8 @@ sub pkit_auth_credential {
   my $login = $model->input('login');
   my $passwd = $model->input('passwd');
 
-  unless ( $login && $passwd ){
-    $model->pkit_gettext_message("You did not fill all of the fields.  Please try again.",
+  unless ($login ne "" && $passwd ne ""){
+    $model->pkit_message("You did not fill all of the fields.  Please try again.",
 		 is_error => 1);
     return;
   }
@@ -85,15 +71,13 @@ sub pkit_auth_credential {
 
   my ($user_id, $dbpasswd) = $dbh->selectrow_array($sql_str, {}, $login);
 
-  unless ($user_id && $dbpasswd && $epasswd eq crypt($dbpasswd,$epasswd)){
-    $model->pkit_gettext_message("Your login/password is invalid. Please try again.",
+  unless ($epasswd eq crypt($dbpasswd,$epasswd)){
+    $model->pkit_message("Your login/password is invalid. Please try again.",
 		is_error => 1);
     return;
   }
 
-  no strict 'refs';
-  my $hash = Digest::MD5::md5_hex(join ':', ${ __PACKAGE__ . '::secret_md5' }, $user_id, $epasswd);
-  use strict 'refs';
+  my $hash = Digest::MD5->md5_hex(join ':', $__PACKAGE__::secret_md5, $user_id, $epasswd);
 
   my $ses_key = {
 		 'user_id'   => $user_id,
@@ -116,9 +100,7 @@ sub pkit_auth_session_key {
 
   # create a new hash and verify that it matches the supplied hash
   # (prevents tampering with the cookie)
-  no strict 'refs';
-  my $newhash = Digest::MD5::md5_hex(join ':', ${ __PACKAGE__ . '::secret_md5' }, $user_id, crypt($epasswd,"pk"));
-  use strict 'refs';
+  my $newhash = Digest::MD5->md5_hex(join ':', $__PACKAGE__::secret_md5, $user_id, crypt($epasswd,"pk"));
 
   return unless $newhash eq $ses_key->{'hash'};
 
@@ -150,15 +132,13 @@ the Model classes for the pagekit.org site.
 It is a good starting point for building your own base class for your
 Model classes.
 
-=head1 AUTHORS
+=head1 AUTHOR
 
 T.J. Mather (tjmather@anidea.com)
 
-Boris Zentner (boris@id9.de)
-
 =head1 COPYRIGHT
 
-Copyright (c) 2000, 2001, 2002 AnIdea, Corp.  All rights Reserved.  PageKit is a trademark
+Copyright (c) 2000, AnIdea, Corp.  All rights Reserved.  PageKit is a trademark
 of AnIdea Corp.
 
 =head1 LICENSE
